/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * packets.h
 * Datagram definitions for libsandnet
 * Copyright (C) 2006 Simon Newton
 */

#ifndef _defined_sandnet_packets_h
#define _defined_sandnet_packets_h

#include <sys/types.h>
#include <stdint.h>
#include <netinet/in.h>
#include <sandnet/common.h>

#define SANDNET_MAC_LENGTH 6
#define SANDNET_NAME_LENGTH 31
#define SANDNET_DMX_LENGTH 512

/*
 * Sandnet opcodes
 */
enum sandnet_packet_type_e {
  SANDNET_AD = 0x0100,
  SANDNET_CTL = 0x0200,
  SANDNET_DMX = 0x0300,
  SANDNET_NAME = 0x0400,
  SANDNET_IDENTIFY = 0x0500,
  SANDNET_PROG = 0x0600,
  SANDNET_LED = 0x0700,
  SANDNET_DMX2 = 0x0a00,
}__attribute__( ( packed ) );

typedef enum sandnet_packet_type_e sandnet_packet_type_t;


struct sandnet_packet_ad_port_s {
  uint8_t    proto;      // protocol
  uint8_t    mode;       // mode
  uint8_t    term;       // terminate
  uint8_t    b;          // ??
  uint8_t    param1;     // subnet
  uint8_t    param2;     // universe
  uint8_t    crap[53];   // ??
}__attribute__( ( packed ) );


/*
 * A Sandnet Advertisment
 */
struct sandnet_ad_s {
  uint16_t op;                          // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH];    // mac address
  uint32_t firmware;                   // firmware version
  struct sandnet_packet_ad_port_s ports[SANDNET_MAX_PORTS];  // ports
  uint8_t  nlen;                       // length of the name field
  uint8_t  name[SANDNET_NAME_LENGTH];  // name field (null terminated)
  uint8_t  magic3[9];                  // magic numbers
  uint8_t  led;                        // flash the led
  uint8_t  magic4;                     // ??
  uint8_t  zero4[64];                  // null
}__attribute__( ( packed ) );

typedef struct sandnet_ad_s sandnet_ad_t;


/*
 * The first of the DMX packets
 */
struct sandnet_dmx_s {
  uint16_t op;             // opcode
  uint8_t  grp;            // group
  uint8_t  uni;            // universe
  uint8_t  port;           // physical port number
  uint8_t  dmx[SANDNET_DMX_LENGTH];  // dmx buffer
}__attribute__( ( packed ) );

typedef struct sandnet_dmx_s sandnet_dmx_t;


/*
 * Changes the port attributes
 */
struct sandnet_pctl_s {
  uint16_t op;                       // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH];  // mac address
  uint8_t  magic[4];                 // ?? seems to change
  struct sandnet_packet_ad_port_s  ports[SANDNET_MAX_PORTS];  // ports
}__attribute__( ( packed ) );

typedef struct sandnet_pctl_s sandnet_pctl_t;


/*
 * Sets the name of the sandnet node
 */
struct sandnet_name_s {
  uint16_t op;                        // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH];   // mac address
  uint8_t  nlen;                      // length of the name field
  uint8_t  name[SANDNET_NAME_LENGTH]; // name field
}__attribute__( ( packed ) );

typedef struct sandnet_name_s sandnet_name_t;


/*
 * identify packet
 * (presumably this flashes the leds or something)
 */
struct sandnet_identify_s {
  uint16_t op;                      // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH]; // mac address
}__attribute__( ( packed ) );

typedef struct sandnet_identify_s sandnet_id_t;


/*
 * ip program packet
 * sets the node's networking parameters
 */
struct sandnet_prog_s {
  uint16_t op;                       // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH];  // mac address
  uint8_t  ip[4];
  uint8_t  dhcp;
  uint8_t  netmask[4];
}__attribute__( ( packed ) );

typedef struct sandnet_prog_s sandnet_prog_t;


/*
 * Turns the led on and off
 */
struct sandnet_led_s {
  uint16_t op;                       // opcode
  uint8_t  mac[SANDNET_MAC_LENGTH];  // mac address
  uint8_t  led;                      // 0x00 off, 0xff on
}__attribute__( ( packed ) );

typedef struct sandnet_led_s sandnet_led_t;



struct sandnet_data_header_s {
  uint16_t op;        // 0x0a00
  uint8_t  grp;       // group
  uint8_t  uni;       // universe
  uint8_t  port;      // physical port number
  uint8_t  zero1[4];  // could be the offset
  uint8_t  two;       // 0x02
  uint16_t length;    // length of data
  uint8_t  data[0];   // dmx data
};

typedef struct sandnet_data_header_s sandnet_data_header_t;


/*
 * DMX data
 */
struct sandnet_data_s {
  sandnet_data_header_t head;
  uint8_t    dmx[SANDNET_DMX_LENGTH];
} __attribute__( ( packed ) );

typedef struct sandnet_data_s sandnet_data_t;


// union of all sandnet packets
typedef union {
  sandnet_ad_t    ad;
  sandnet_pctl_t  pctl;
  sandnet_dmx_t   dmx;
  sandnet_name_t  name;
  sandnet_id_t    id;
  sandnet_prog_t  prog;
  sandnet_led_t   led;
  sandnet_data_t  data;
} sandnet_packet_union_t;


// a packet, containing data, length, type and a src/dst address
typedef struct {
  int length;
  struct in_addr from;
  struct in_addr to;
  sandnet_packet_union_t data;
} sandnet_packet_t;

typedef sandnet_packet_t *sandnet_packet;

#endif
